"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _fileMutex = _interopRequireDefault(require("../../../../../core/playwright/helpers/fileMutex"));
var _lockfile = _interopRequireDefault(require("lockfile"));
var _path = _interopRequireDefault(require("path"));
var _fs = _interopRequireDefault(require("fs"));
jest.mock('lockfile');
jest.mock('fs');
describe('FileMutex', () => {
  const mockDirectory = '/tmp';
  const mockFileName = 'test_user';
  const mockTimeout = {
    wait: 5000,
    retries: 5,
    stale: 60000
  };
  let fileMutex;
  beforeEach(() => {
    fileMutex = new _fileMutex.default(mockDirectory, mockFileName, mockTimeout);
  });
  afterEach(() => {
    jest.clearAllMocks();
  });
  describe('acquire', () => {
    it('should resolve when lockfile.lock succeeds and call getLockFilePath correctly', async () => {
      const expectedPath = _path.default.resolve(_path.default.join(mockDirectory, `./${mockFileName}.lock`));
      _lockfile.default.lock.mockImplementation((lockPath, options, callback) => {
        expect(lockPath).toBe(expectedPath);
        expect(options).toEqual(mockTimeout);
        callback(null);
      });
      await expect(fileMutex.acquire()).resolves.toBeUndefined();
      expect(_lockfile.default.lock).toHaveBeenCalledWith(expectedPath, mockTimeout, expect.any(Function));
    });
    it('should reject when lockfile.lock fails', async () => {
      const mockError = new Error('Lock failed');
      _lockfile.default.lock.mockImplementation((lockPath, options, callback) => {
        expect(lockPath).toBe(_path.default.resolve(_path.default.join(mockDirectory, `./${mockFileName}.lock`)));
        expect(options).toEqual(mockTimeout);
        callback(mockError);
      });
      await expect(fileMutex.acquire()).rejects.toThrow('Lock failed');
      expect(_lockfile.default.lock).toHaveBeenCalled();
    });
    it('should create the directory if it does not exist and acquire the lock', async () => {
      _fs.default.existsSync.mockReturnValue(false);
      _fs.default.mkdirSync.mockImplementation((dir, options) => {
        expect(dir).toBe(mockDirectory);
        expect(options).toEqual({
          recursive: true
        });
      });
      _lockfile.default.lock.mockImplementation((lockPath, options, callback) => {
        expect(lockPath).toBe(_path.default.resolve(_path.default.join(mockDirectory, `./${mockFileName}.lock`)));
        expect(options).toEqual(mockTimeout);
        callback(null);
      });
      await expect(fileMutex.acquire()).resolves.toBeUndefined();
      expect(_fs.default.existsSync).toHaveBeenCalledWith(mockDirectory);
      expect(_fs.default.mkdirSync).toHaveBeenCalledWith(mockDirectory, {
        recursive: true
      });
      expect(_lockfile.default.lock).toHaveBeenCalled();
    });
    it('should not create the directory if it already exists', async () => {
      _fs.default.existsSync.mockReturnValue(true);
      _lockfile.default.lock.mockImplementation((lockPath, options, callback) => {
        callback(null);
      });
      await expect(fileMutex.acquire()).resolves.toBeUndefined();
      expect(_fs.default.mkdirSync).not.toHaveBeenCalled();
      expect(_lockfile.default.lock).toHaveBeenCalled();
    });
  });
  describe('release', () => {
    it('should resolve when lockfile.unlock succeeds and call getLockFilePath correctly', async () => {
      const expectedPath = _path.default.resolve(_path.default.join(mockDirectory, `./${mockFileName}.lock`));
      _lockfile.default.unlock.mockImplementation((path, callback) => {
        callback(null);
      });
      await expect(fileMutex.release()).resolves.toBeUndefined();
      expect(_lockfile.default.unlock).toHaveBeenCalledWith(expectedPath, expect.any(Function));
    });
    it('should reject when lockfile.unlock fails', async () => {
      const mockError = new Error('Unlock failed');
      _lockfile.default.unlock.mockImplementation((path, callback) => {
        callback(mockError);
      });
      await expect(fileMutex.release()).rejects.toThrow('Unlock failed');
      expect(_lockfile.default.unlock).toHaveBeenCalled();
    });
  });
});